@@ -12,6 +12,7 @@ def generate_markdown_documentation(
     model_prefix="Model",
     fields_prefix="Fields",
     documentation_with_field_description=True,
+    ordered_json_mode=False,
 ) -> str:
     """
     Generate markdown documentation for a list of Pydantic models.
@@ -25,13 +26,13 @@ def generate_markdown_documentation(
     Returns:
         str: Generated text documentation.
     """
-    documentation = "## Functions\n\n"
+    documentation = ""
     pyd_models = [(model, True) for model in pydantic_models]
     for model, add_prefix in pyd_models:
         if add_prefix:
-            documentation += f"### {model_prefix}: {model.__name__}\n"
+            documentation += f"### {model_prefix} '{model.__name__}'\n"
         else:
-            documentation += f"### {model.__name__}\n"
+            documentation += f"### '{model.__name__}'\n"
 
         # Handling multi-line model description with proper indentation
 
@@ -45,10 +46,11 @@ def generate_markdown_documentation(
 
         if add_prefix:
             # Indenting the fields section
-            documentation += f"{fields_prefix}:\n"
+            documentation += f"#### {fields_prefix}\n"
         else:
-            documentation += f"Fields:\n"
+            documentation += f"#### Fields\n"
         if isclass(model) and issubclass(model, BaseModel):
+            count = 1
             for name, field_type in model.__annotations__.items():
                 # if name == "markdown_code_block":
                 #    continue
@@ -64,16 +66,18 @@ def generate_markdown_documentation(
                         ):
                             pyd_models.append((element_type, False))
                 documentation += generate_field_markdown(
-                    name,
+                    name
+                    if not ordered_json_mode
+                    else "{:03}".format(count) + "_" + name,
                     field_type,
                     model,
                     documentation_with_field_description=documentation_with_field_description,
                 )
             if add_prefix:
-                if documentation.endswith(f"{fields_prefix}:\n"):
+                if documentation.endswith(f"#### {fields_prefix}\n"):
                     documentation += "none\n"
             else:
-                if documentation.endswith("Fields:\n"):
+                if documentation.endswith("#### Fields\n"):
                     documentation += "none\n"
             documentation += "\n"
 
@@ -109,13 +113,14 @@ def generate_field_markdown(
     Returns:
         str: Generated text documentation for the field.
     """
-    indent = "  " * depth
+    indent = ""
 
     field_info = model.model_fields.get(field_name)
     field_description = (
         field_info.description if field_info and field_info.description else ""
     )
-
+    is_enum = False
+    enum_values = None
     if get_origin(field_type) == list:
         element_type = get_args(field_type)[0]
         field_text = (
@@ -141,8 +146,8 @@ def generate_field_markdown(
 
     elif issubclass(field_type, Enum):
         enum_values = [f"'{str(member.value)}'" for member in field_type]
-
-        field_text = f"{indent}{field_name} ({' or '.join(enum_values)})"
+        is_enum = True
+        field_text = f"{indent}{field_name}"
         if field_description != "":
             field_text += ": "
         else:
@@ -158,9 +163,13 @@ def generate_field_markdown(
     if not documentation_with_field_description:
         return field_text
 
-    if field_description != "":
+    if is_enum:
+
+        field_text += field_description + f" Can be one of the following values: {' or '.join(enum_values)}" + "\n"
+    elif field_description != "":
         field_text += field_description + "\n"
 
+
     # Check for and include field-specific examples if available
     if (
         hasattr(model, "Config")
